@@ -1,20 +1,29 @@
 from pkg_resources import get_distribution, DistributionNotFound
 import os.path
+from .core import mass  # noqa: F401
 from .stump import stump  # noqa: F401
 from .stumped import stumped  # noqa: F401
 from .mstump import mstump, subspace  # noqa: F401
 from .mstumped import mstumped  # noqa: F401
 from .aamp import aamp  # noqa: F401
 from .aamped import aamped  # noqa: F401
+from .maamp import maamp, maamp_subspace  # noqa: F401
+from .maamped import maamped  # noqa: F401
 from .aampi import aampi  # noqa: F401
 from .chains import atsc, allc  # noqa: F401
 from .floss import floss, fluss  # noqa: F401
 from .ostinato import ostinato, ostinatoed  # noqa: F401
 from .aamp_ostinato import aamp_ostinato, aamp_ostinatoed  # noqa: F401
-from .scrump import scrump  # noqa: F401
+from .scrump import scrump, prescrump  # noqa: F401
+from .scraamp import scraamp, prescraamp  # noqa: F401
 from .stumpi import stumpi  # noqa: F401
 from .mpdist import mpdist, mpdisted  # noqa: F401
 from .aampdist import aampdist, aampdisted  # noqa: F401
+from .motifs import motifs, match  # noqa: F401
+from .aamp_motifs import aamp_motifs, aamp_match  # noqa: F401
+from .snippets import snippets  # noqa: F401
+from .aampdist_snippets import aampdist_snippets  # noqa: F401
+from .stimp import stimp, stimped  # noqa: F401
 from numba import cuda
 
 if cuda.is_available():
@@ -24,6 +33,7 @@ if cuda.is_available():
     from .gpu_aamp_ostinato import gpu_aamp_ostinato  # noqa: F401
     from .gpu_mpdist import gpu_mpdist  # noqa: F401
     from .gpu_aampdist import gpu_aampdist  # noqa: F401
+    from .gpu_stimp import gpu_stimp  # noqa: F401
 else:  # pragma: no cover
     from .core import _gpu_stump_driver_not_found as gpu_stump  # noqa: F401
     from .core import _gpu_aamp_driver_not_found as gpu_aamp  # noqa: F401
@@ -33,6 +43,7 @@ else:  # pragma: no cover
     )  # noqa: F401
     from .core import _gpu_mpdist_driver_not_found as gpu_mpdist  # noqa: F401
     from .core import _gpu_aampdist_driver_not_found as gpu_aampdist  # noqa: F401
+    from .core import _gpu_stimp_driver_not_found as gpu_stimp  # noqa: F401
     import ast
     import pathlib
 
@@ -126,6 +137,21 @@ else:  # pragma: no cover
         if fd.name == "gpu_aampdist":
             gpu_aampdist.__doc__ = ast.get_docstring(fd)
 
+    # Fix GPU-STIMP Docs
+    gpu_stimp.__doc__ = ""
+    filepath = pathlib.Path(__file__).parent / "gpu_stimp.py"
+
+    file_contents = ""
+    with open(filepath, encoding="utf8") as f:
+        file_contents = f.read()
+    module = ast.parse(file_contents)
+    function_definitions = [
+        node for node in module.body if isinstance(node, ast.FunctionDef)
+    ]
+    for fd in function_definitions:
+        if fd.name == "gpu_stimp":
+            gpu_aampdist.__doc__ = ast.get_docstring(fd)
+
 try:
     _dist = get_distribution("stumpy")
     # Normalize case for Windows systems
