@@ -11,6 +11,9 @@ def atsc(IL, IR, j):
     """
     Compute the anchored time series chain (ATSC)
 
+    Note that since the matrix profile indices, `IL` and `IR`, are pre-computed,
+    this function is agnostic to subsequence normalization.
+
     Parameters
     ----------
     IL : ndarray
@@ -24,7 +27,7 @@ def atsc(IL, IR, j):
 
     Returns
     -------
-    output : ndarray
+    out : ndarray
         Anchored time series chain for index, `j`
 
     Notes
@@ -46,13 +49,18 @@ def atsc(IL, IR, j):
             j = IR[j]
             C.append(j)
 
-    return np.array(list(C), dtype=np.int64)
+    out = np.array(list(C), dtype=np.int64)
+
+    return out
 
 
 def allc(IL, IR):
     """
     Compute the all-chain set (ALLC)
 
+    Note that since the matrix profile indices, `IL` and `IR`, are pre-computed,
+    this function is agnostic to subsequence normalization.
+
     Parameters
     ----------
     IL : ndarray
