@@ -2,7 +2,10 @@
 # Copyright 2019 TD Ameritrade. Released under the terms of the 3-Clause BSD license.  # noqa: E501
 # STUMPY is a trademark of TD Ameritrade IP Company, Inc. All rights reserved.
 
-from functools import partial
+import logging
+import functools
+import inspect
+
 import numpy as np
 from numba import njit, prange
 from scipy.signal import convolve
@@ -17,6 +20,126 @@ try:
 except ImportError:
     pass
 
+logger = logging.getLogger(__name__)
+
+
+def _compare_parameters(norm, non_norm, exclude=None):
+    """
+    Compare if the parameters in `norm` and `non_norm` are the same
+
+    Parameters
+    ----------
+    norm : object
+        The normalized function (or class) that is complementary to the
+        non-normalized function (or class)
+
+    non_norm : object
+        The non-normalized function (or class) that is complementary to the
+        z-normalized function (or class)
+
+    exclude : list
+        A list of parameters to exclude for the comparison
+
+    Returns
+    -------
+    is_same_params : bool
+        `True` if parameters from both `norm` and `non-norm` are the same. `False`
+        otherwise.
+    """
+    norm_params = list(inspect.signature(norm).parameters.keys())
+    non_norm_params = list(inspect.signature(non_norm).parameters.keys())
+
+    if exclude is not None:
+        for param in exclude:
+            if param in norm_params:
+                norm_params.remove(param)
+            if param in non_norm_params:
+                non_norm_params.remove(param)
+
+    is_same_params = set(norm_params) == set(non_norm_params)
+    if not is_same_params:
+        if exclude is not None:
+            logger.warning(f"Excluding `{exclude}` parameters, ")
+        logger.warning(f"`{norm}`: ({norm_params}) and ")
+        logger.warning(f"`{non_norm}`: ({non_norm_params}) ")
+        logger.warning("have different parameters.")
+
+    return is_same_params
+
+
+def non_normalized(non_norm, exclude=None, replace=None):
+    """
+    Decorator for swapping a z-normalized function (or class) for its complementary
+    non-normalized function (or class) as defined by `non_norm`. This requires that
+    the z-normalized function (or class) has a `normalize` parameter.
+
+    With the exception of `normalize` parameter, the `non_norm` function (or class)
+    must have the same siganture as the `norm` function (or class) signature in order
+    to be compatible. Please use a combination of the `exclude` and/or `replace`
+    parameters when necessary.
+
+    ```
+    def non_norm_func(Q, T, A):
+        ...
+        return
+
+
+    @non_normalized(
+        non_norm_func,
+        exclude=["normalize", "A", "B"],
+        replace={"A": None},
+    )
+    def norm_func(Q, T, B=None, normalize=True):
+        ...
+        return
+    ```
+
+    Parameters
+    ----------
+    non_norm : object
+        The non-normalized function (or class) that is complementary to the
+        z-normalized function (or class)
+
+    exclude : list, default None
+        A list of function (or class) parameter names to exclude when comparing the
+        function (or class) signatures
+
+    replace : dict, default None
+        A dictionary of function (or class) parameter key-value pairs. Each key that
+        is found as a parameter name in the `norm` function (or class) will be replaced
+        by its corresponding or complementary parameter name in the `non_norm` function
+        (or class).
+
+    Returns
+    -------
+    outer_wrapper : object
+        The desired z-normalized/non-normalized function (or class)
+    """
+    if exclude is None:
+        exclude = ["normalize"]
+
+    @functools.wraps(non_norm)
+    def outer_wrapper(norm):
+        @functools.wraps(norm)
+        def inner_wrapper(*args, **kwargs):
+            is_same_params = _compare_parameters(norm, non_norm, exclude=exclude)
+            if not is_same_params or kwargs.get("normalize", True):
+                return norm(*args, **kwargs)
+            else:
+                kwargs = {k: v for k, v in kwargs.items() if k != "normalize"}
+                if replace is not None:
+                    for k, v in replace.items():
+                        if k in kwargs.keys():
+                            if v is None:
+                                _ = kwargs.pop(k)
+                            else:
+                                kwargs[v] = kwargs.pop(k)
+                return non_norm(*args, **kwargs)
+
+        return inner_wrapper
+
+    return outer_wrapper
+
 
 def driver_not_found(*args, **kwargs):  # pragma: no cover
     """
@@ -67,6 +190,13 @@ def _gpu_aampdist_driver_not_found(*args, **kwargs):  # pragma: no cover
     driver_not_found()
 
 
+def _gpu_stimp_driver_not_found(*args, **kwargs):  # pragma: no cover
+    """
+    Dummy function to raise CudaSupportError driver not found error.
+    """
+    driver_not_found()
+
+
 def get_pkg_name():  # pragma: no cover
     """
     Return package name.
@@ -214,7 +344,13 @@ def are_arrays_equal(a, b):  # pragma: no cover
     if id(a) == id(b):
         return True
 
-    return np.array_equal(a, b, equal_nan=True)
+    # For numpy >= 1.19
+    # return np.array_equal(a, b, equal_nan=True)
+
+    if a.shape != b.shape:
+        return False
+
+    return ((a == b) | (np.isnan(a) & np.isnan(b))).all()
 
 
 def are_distances_too_small(a, threshold=10e-6):  # pragma: no cover
@@ -244,6 +380,34 @@ def are_distances_too_small(a, threshold=10e-6):  # pragma: no cover
     return False
 
 
+def get_max_window_size(n):
+    """
+    Get the maximum window size for a self-join
+
+    Parameters
+    ----------
+    n : int
+        The length of the time series
+
+    Returns
+    -------
+    max_m : int
+        The maximum window size allowed given `config.STUMPY_EXCL_ZONE_DENOM`
+    """
+    max_m = (
+        int(
+            n
+            - np.floor(
+                (n + (config.STUMPY_EXCL_ZONE_DENOM - 1))
+                // (config.STUMPY_EXCL_ZONE_DENOM + 1)
+            )
+        )
+        - 1
+    )
+
+    return max_m
+
+
 def check_window_size(m, max_size=None):
     """
     Check the window size and ensure that it is greater than or equal to 3 and, if
@@ -429,7 +593,7 @@ def welford_nanstd(a, w=None):
     if w is None:
         w = a.shape[0]
 
-    return np.sqrt(welford_nanvar(a, w))
+    return np.sqrt(np.clip(welford_nanvar(a, w), a_min=0, a_max=None))
 
 
 def rolling_nanstd(a, w):
@@ -812,6 +976,143 @@ def calculate_distance_profile(m, QT, μ_Q, σ_Q, M_T, Σ_T):
     return np.sqrt(D_squared)
 
 
+@njit(fastmath=True)
+def _mass_absolute(Q_squared, T_squared, QT):
+    """
+    A Numba JIT compiled algorithm for computing the non-normalized distance profile
+    using the MASS absolute algorithm.
+
+    Parameters
+    ----------
+    Q_squared : ndarray
+        Squared query array or subsequence
+
+    T_squared : ndarray
+        Squared time series or sequence
+
+    QT : ndarray
+        Sliding window dot product of `Q` and `T`
+
+    Returns
+    -------
+    output : ndarray
+        Unnormalized distance profile
+
+    Notes
+    -----
+    `See Mueen's Absolute Algorithm for Similarity Search \
+    <https://www.cs.unm.edu/~mueen/MASS_absolute.m>`__
+    """
+    D = Q_squared + T_squared - 2 * QT
+    D[D < config.STUMPY_D_SQUARED_THRESHOLD] = 0.0
+
+    return np.sqrt(D)
+
+
+def mass_absolute(Q, T, T_subseq_isfinite=None, T_squared=None):
+    """
+    Compute the non-normalized distance profile (i.e., without z-normalization) using
+    the "MASS absolute" algorithm. This is a convenience wrapper around the Numba JIT
+    compiled `_mass_absolute` function.
+
+    Parameters
+    ----------
+    Q : ndarray
+        Query array or subsequence
+
+    T : ndarray
+        Time series or sequence
+
+    T_subseq_isfinite : ndarray, default None
+        A boolean array that indicates whether a subsequence in `T` contains a
+        `np.nan`/`np.inf` value (False)
+
+    T_squared : ndarray, default None
+        Squared time series or sequence
+
+    Returns
+    -------
+    output : ndarray
+        Unnormalized Distance profile
+
+    Notes
+    -----
+    `See Mueen's Absolute Algorithm for Similarity Search \
+    <https://www.cs.unm.edu/~mueen/MASS_absolute.m>`__
+    """
+    Q = Q.copy()
+    Q = np.asarray(Q)
+    check_dtype(Q)
+    m = Q.shape[0]
+
+    if Q.ndim == 2 and Q.shape[1] == 1:  # pragma: no cover
+        Q = Q.flatten()
+
+    if Q.ndim != 1:  # pragma: no cover
+        raise ValueError(f"Q is {Q.ndim}-dimensional and must be 1-dimensional. ")
+
+    T = T.copy()
+    T = np.asarray(T)
+    check_dtype(T)
+    n = T.shape[0]
+
+    if T.ndim == 2 and T.shape[1] == 1:  # pragma: no cover
+        T = T.flatten()
+
+    if T.ndim != 1:  # pragma: no cover
+        raise ValueError(f"T is {T.ndim}-dimensional and must be 1-dimensional. ")
+
+    if m > n:  # pragma: no cover
+        raise ValueError(
+            f"The length of `Q` ({len(Q)}) must be less than or equal to "
+            f"the length of `T` ({len(T)}). "
+        )
+
+    distance_profile = np.empty(n - m + 1)
+    if np.any(~np.isfinite(Q)):
+        distance_profile[:] = np.inf
+    else:
+        if T_subseq_isfinite is None:
+            T, T_subseq_isfinite = preprocess_non_normalized(T, m)
+        QT = sliding_dot_product(Q, T)
+        Q_squared = np.sum(Q * Q)
+        if T_squared is None:
+            T_squared = np.sum(rolling_window(T * T, m), axis=-1)
+        distance_profile[:] = _mass_absolute(Q_squared, T_squared, QT)
+        distance_profile[~T_subseq_isfinite] = np.inf
+
+    return distance_profile
+
+
+def _mass_absolute_distance_matrix(Q, T, m, distance_matrix):
+    """
+    Compute the full non-normalized (i.e., without z-normalization) distance matrix
+    between all of the subsequences of `Q` and `T` using the MASS absolute algorithm
+
+    Parameters
+    ----------
+    Q : ndarray
+        Query array
+
+    T : ndarray
+        Time series or sequence
+
+    m : int
+        Window size
+
+    distance_matrix : ndarray
+        The full output distance matrix. This is mandatory since it may be reused.
+
+    Returns
+    -------
+    None
+    """
+    k, l = distance_matrix.shape
+
+    for i in range(k):
+        distance_matrix[i, :] = mass_absolute(Q[i : i + m], T)
+
+
 def mueen_calculate_distance_profile(Q, T):
     """
     Compute the mueen distance profile
@@ -926,10 +1227,16 @@ def _mass(Q, T, QT, μ_Q, σ_Q, M_T, Σ_T):
     return calculate_distance_profile(m, QT, μ_Q, σ_Q, M_T, Σ_T)
 
 
-def mass(Q, T, M_T=None, Σ_T=None):
+@non_normalized(
+    mass_absolute,
+    exclude=["normalize", "M_T", "Σ_T", "T_subseq_isfinite", "T_squared"],
+    replace={"M_T": None, "Σ_T": None},
+)
+def mass(Q, T, M_T=None, Σ_T=None, normalize=True):
     """
-    Compute the distance profile using the MASS algorithm. This is a convenience
-    wrapper around the Numba JIT compiled `_mass` function.
+    Compute the distance profile using the MASS algorithm
+
+    This is a convenience wrapper around the Numba JIT compiled `_mass` function.
 
     Parameters
     ----------
@@ -945,9 +1252,14 @@ def mass(Q, T, M_T=None, Σ_T=None):
     Σ_T : ndarray, default None
         Sliding standard deviation of `T`
 
+    normalize : bool, default True
+        When set to `True`, this z-normalizes subsequences prior to computing distances.
+        Otherwise, this function gets re-routed to its complementary non-normalized
+        equivalent set in the `@core.non_normalized` function decorator.
+
     Returns
     -------
-    output : ndarray
+    distance_profile : ndarray
         Distance profile
 
     Notes
@@ -1036,135 +1348,6 @@ def _mass_distance_matrix(Q, T, m, distance_matrix):
         distance_matrix[i, :] = mass(Q[i : i + m], T, M_T, Σ_T)
 
 
-@njit(fastmath=True)
-def _mass_absolute(Q_squared, T_squared, QT):
-    """
-    A Numba JIT compiled algorithm for computing the non-normalized distance profile
-    using the MASS absolute algorithm.
-
-    Parameters
-    ----------
-    Q_squared : ndarray
-        Squared query array or subsequence
-
-    T_squared : ndarray
-        Squared time series or sequence
-
-    QT : ndarray
-        Sliding window dot product of `Q` and `T`
-
-    Returns
-    -------
-    output : ndarray
-        Unnormalized distance profile
-
-    Notes
-    -----
-    `See Mueen's Absolute Algorithm for Similarity Search \
-    <https://www.cs.unm.edu/~mueen/MASS_absolute.m>`__
-    """
-    D = Q_squared + T_squared - 2 * QT
-    D[D < config.STUMPY_D_SQUARED_THRESHOLD] = 0.0
-
-    return np.sqrt(D)
-
-
-def mass_absolute(Q, T):
-    """
-    Compute the non-normalized distance profile (i.e., without z-normalization) using
-    the "MASS absolute" algorithm. This is a convenience wrapper around the Numba JIT
-    compiled `_mass_absolute` function.
-
-    Parameters
-    ----------
-    Q : ndarray
-        Query array or subsequence
-
-    T : ndarray
-        Time series or sequence
-
-    Returns
-    -------
-    output : ndarray
-        Unnormalized Distance profile
-
-    Notes
-    -----
-    `See Mueen's Absolute Algorithm for Similarity Search \
-    <https://www.cs.unm.edu/~mueen/MASS_absolute.m>`__
-    """
-    Q = Q.copy()
-    Q = np.asarray(Q)
-    check_dtype(Q)
-    m = Q.shape[0]
-
-    if Q.ndim == 2 and Q.shape[1] == 1:  # pragma: no cover
-        Q = Q.flatten()
-
-    if Q.ndim != 1:  # pragma: no cover
-        raise ValueError(f"Q is {Q.ndim}-dimensional and must be 1-dimensional. ")
-
-    T = T.copy()
-    T = np.asarray(T)
-    check_dtype(T)
-    n = T.shape[0]
-
-    if T.ndim == 2 and T.shape[1] == 1:  # pragma: no cover
-        T = T.flatten()
-
-    if T.ndim != 1:  # pragma: no cover
-        raise ValueError(f"T is {T.ndim}-dimensional and must be 1-dimensional. ")
-
-    if m > n:  # pragma: no cover
-        raise ValueError(
-            f"The length of `Q` ({len(Q)}) must be less than or equal to "
-            f"the length of `T` ({len(T)}). "
-        )
-
-    distance_profile = np.empty(n - m + 1)
-    if np.any(~np.isfinite(Q)):
-        distance_profile[:] = np.inf
-    else:
-        T_subseq_isfinite = rolling_isfinite(T, m)
-        T[~np.isfinite(T)] = 0.0
-        QT = sliding_dot_product(Q, T)
-        Q_squared = np.sum(Q * Q)
-        T_squared = np.sum(rolling_window(T * T, m), axis=1)
-        distance_profile[:] = _mass_absolute(Q_squared, T_squared, QT)
-        distance_profile[~T_subseq_isfinite] = np.inf
-
-    return distance_profile
-
-
-def _mass_absolute_distance_matrix(Q, T, m, distance_matrix):
-    """
-    Compute the full non-normalized (i.e., without z-normalization) distance matrix
-    between all of the subsequences of `Q` and `T` using the MASS absolute algorithm
-
-    Parameters
-    ----------
-    Q : ndarray
-        Query array
-
-    T : ndarray
-        Time series or sequence
-
-    m : int
-        Window size
-
-    distance_matrix : ndarray
-        The full output distance matrix. This is mandatory since it may be reused.
-
-    Returns
-    -------
-    None
-    """
-    k, l = distance_matrix.shape
-
-    for i in range(k):
-        distance_matrix[i, :] = mass_absolute(Q[i : i + m], T)
-
-
 def _get_QT(start, T_A, T_B, m):
     """
     Compute the sliding dot product between the query, `T_A`, (from
@@ -1297,9 +1480,8 @@ def preprocess_non_normalized(T, m):
     check_dtype(T)
     check_window_size(m, max_size=T.shape[-1])
 
-    T[np.isinf(T)] = np.nan
     T_subseq_isfinite = rolling_isfinite(T, m)
-    T[np.isnan(T)] = 0
+    T[~np.isfinite(T)] = 0.0
 
     return T, T_subseq_isfinite
 
@@ -1469,25 +1651,26 @@ def _get_array_ranges(a, n_chunks, truncate=False):
         contains the stop indices.
     """
     array_ranges = np.zeros((n_chunks, 2), np.int64)
-    cumsum = a.cumsum() / a.sum()
-    insert = np.linspace(0, 1, n_chunks + 1)[1:-1]
-    idx = 1 + np.searchsorted(cumsum, insert)
-    array_ranges[1:, 0] = idx  # Fill the first column with start indices
-    array_ranges[:-1, 1] = idx  # Fill the second column with exclusive stop indices
-    array_ranges[-1, 1] = a.shape[0]  # Handle the stop index for the final chunk
+    if a.shape[0] > 0 and n_chunks > 0:
+        cumsum = a.cumsum() / a.sum()
+        insert = np.linspace(0, 1, n_chunks + 1)[1:-1]
+        idx = 1 + np.searchsorted(cumsum, insert)
+        array_ranges[1:, 0] = idx  # Fill the first column with start indices
+        array_ranges[:-1, 1] = idx  # Fill the second column with exclusive stop indices
+        array_ranges[-1, 1] = a.shape[0]  # Handle the stop index for the final chunk
 
-    diff_idx = np.diff(idx)
-    if np.any(diff_idx == 0):
-        row_truncation_idx = np.argmin(diff_idx) + 2
-        array_ranges[row_truncation_idx:, 0] = a.shape[0]
-        array_ranges[row_truncation_idx - 1 :, 1] = a.shape[0]
-        if truncate:
-            array_ranges = array_ranges[:row_truncation_idx]
+        diff_idx = np.diff(idx)
+        if np.any(diff_idx == 0):
+            row_truncation_idx = np.argmin(diff_idx) + 2
+            array_ranges[row_truncation_idx:, 0] = a.shape[0]
+            array_ranges[row_truncation_idx - 1 :, 1] = a.shape[0]
+            if truncate:
+                array_ranges = array_ranges[:row_truncation_idx]
 
     return array_ranges
 
 
-def rolling_isfinite(a, w):
+def _rolling_isfinite_1d(a, w):
     """
     Determine if all elements in each rolling window `isfinite`
 
@@ -1522,6 +1705,31 @@ def rolling_isfinite(a, w):
     return a_isfinite[: a_isfinite.shape[0] - w + 1]
 
 
+def rolling_isfinite(a, w):
+    """
+    Compute the rolling `isfinite` for 1-D and 2-D arrays.
+
+    This a convenience wrapper around `_rolling_isfinite_1d`.
+
+    Parameters
+    ----------
+    a : ndarray
+        The input array
+
+    w : ndarray
+        The rolling window size
+
+    Returns
+    -------
+    output : ndarray
+        Rolling window nanmax.
+    """
+    axis = a.ndim - 1  # Account for rolling
+    return np.apply_along_axis(
+        lambda a_row, w: _rolling_isfinite_1d(a_row, w), axis=axis, arr=a, w=w
+    )
+
+
 def _get_partial_mp_func(mp_func, dask_client=None, device_id=None):
     """
     A convenience function for creating a `functools.partial` matrix profile function
@@ -1552,10 +1760,78 @@ def _get_partial_mp_func(mp_func, dask_client=None, device_id=None):
         `device_id` into `functools.partial` function where possible
     """
     if dask_client is not None:
-        partial_mp_func = partial(mp_func, dask_client)
+        partial_mp_func = functools.partial(mp_func, dask_client)
     elif device_id is not None:
-        partial_mp_func = partial(mp_func, device_id=device_id)
+        partial_mp_func = functools.partial(mp_func, device_id=device_id)
     else:
         partial_mp_func = mp_func
 
     return partial_mp_func
+
+
+def _jagged_list_to_array(a, fill_value, dtype):
+    """
+    Fits a 2d jagged list into a 2d numpy array of the specified dtype.
+    The resulting array will have a shape of (len(a), l), where l is the length
+    of the longest list in a. All other lists will be padded with `fill_value`.
+
+    Example:
+    [[2, 1, 1], [0]] with a fill value of -1 will become
+    np.array([[2, 1, 1], [0, -1, -1]])
+
+    Parameters
+    ----------
+    a : list
+        Jagged list (list-of-lists) to be converted into a ndarray.
+
+    fill_value : int or float
+        Missing entries will be filled with this value.
+
+    dtype : dtype
+        The desired data-type for the array.
+
+    Return
+    ------
+    out : ndarray
+        The resuling ndarray of dtype `dtype`.
+    """
+    if not a:
+        return np.array([[]])
+
+    max_length = max([len(row) for row in a])
+
+    out = np.full((len(a), max_length), fill_value, dtype=dtype)
+
+    for i, row in enumerate(a):
+        out[i, : row.size] = row
+
+    return out
+
+
+def _get_mask_slices(mask):
+    """
+    For a boolean vector mask, return the (inclusive) start and (exclusive) stop
+    indices where the mask is `True`.
+
+    Parameters
+    ----------
+    mask: ndarray
+        A boolean 1D array
+
+    Returns
+    -------
+    slices: ndarray
+        slices of indices where the mask is True. Each slice has a size of two:
+        The first number is the start index (inclusive)
+        The second number is the end index (exclusive)
+
+    """
+    m1 = np.r_[0, mask]
+    m2 = np.r_[mask, 0]
+
+    (starts,) = np.where(~m1 & m2)
+    (ends,) = np.where(m1 & ~m2)
+
+    slices = np.c_[starts, ends]
+
+    return slices
