@@ -5,11 +5,13 @@
 # import numpy as np
 # import math
 
-from . import gpu_stump
+from . import core, gpu_stump
 from .mpdist import _mpdist
+from .gpu_aampdist import gpu_aampdist
 
 
-def gpu_mpdist(T_A, T_B, m, percentage=0.05, k=None, device_id=0):
+@core.non_normalized(gpu_aampdist)
+def gpu_mpdist(T_A, T_B, m, percentage=0.05, k=None, device_id=0, normalize=True):
     """
     Compute the z-normalized matrix profile distance (MPdist) measure between any two
     time series with one or more GPU devices
@@ -46,6 +48,11 @@ def gpu_mpdist(T_A, T_B, m, percentage=0.05, k=None, device_id=0):
         computation. A list of all valid device ids can be obtained by
         executing `[device.id for device in numba.cuda.list_devices()]`.
 
+    normalize : bool, default True
+        When set to `True`, this z-normalizes subsequences prior to computing distances.
+        Otherwise, this function gets re-routed to its complementary non-normalized
+        equivalent set in the `@core.non_normalized` function decorator.
+
     Returns
     -------
     MPdist : float
@@ -58,4 +65,6 @@ def gpu_mpdist(T_A, T_B, m, percentage=0.05, k=None, device_id=0):
 
     See Section III
     """
-    return _mpdist(T_A, T_B, m, percentage, k, device_id=device_id, mp_func=gpu_stump)
+    MPdist = _mpdist(T_A, T_B, m, percentage, k, device_id=device_id, mp_func=gpu_stump)
+
+    return MPdist
