@@ -9,7 +9,8 @@ from scipy.stats import norm
 from numba import njit, prange
 from functools import lru_cache
 
-from . import core
+from . import core, config
+from .maamp import maamp, maamp_subspace
 
 logger = logging.getLogger(__name__)
 
@@ -192,7 +193,61 @@ def _discretize(a, bins, right=True):
     return np.digitize(a, bins, right=right)
 
 
-def subspace(T, m, subseq_idx, nn_idx, k, include=None, discords=False):
+def _subspace(D, k, include=None, discords=False):
+    """
+    Compute the k-dimensional matrix profile subspace for a given subsequence index and
+    its nearest neighbor index
+
+    Parameters
+    ----------
+    D : ndarray
+        The multi-dimensional distance profile
+
+    k : int
+        The subset number of dimensions out of `D = T.shape[0]`-dimensions to return
+        the subspace for
+
+    include : ndarray, default None
+        A list of (zero-based) indices corresponding to the dimensions in `T` that
+        must be included in the constrained multidimensional motif search.
+        For more information, see Section IV D in:
+
+        `DOI: 10.1109/ICDM.2017.66 \
+        <https://www.cs.ucr.edu/~eamonn/Motif_Discovery_ICDM.pdf>`__
+
+    discords : bool, default False
+        When set to `True`, this reverses the distance profile to favor discords rather
+        than motifs. Note that indices in `include` are still maintained and respected.
+
+    Returns
+    -------
+        S : ndarray
+        An array of that contains the `k`th-dimensional subspace for the subsequence
+        with index equal to `motif_idx`
+    """
+    if discords:
+        sorted_idx = D[::-1].argsort(axis=0, kind="mergesort")
+    else:
+        sorted_idx = D.argsort(axis=0, kind="mergesort")
+
+    # `include` processing occur here since we are dealing with indices, not distances
+    if include is not None:
+        include = _preprocess_include(include)
+        mask = np.in1d(sorted_idx, include)
+        include_idx = mask.nonzero()[0]
+        exclude_idx = (~mask).nonzero()[0]
+        sorted_idx[: include_idx.shape[0]], sorted_idx[include_idx.shape[0] :] = (
+            sorted_idx[include_idx],
+            sorted_idx[exclude_idx],
+        )
+
+    S = sorted_idx[: k + 1]
+
+    return S
+
+
+@core.non_normalized(maamp_subspace)
+def subspace(T, m, subseq_idx, nn_idx, k, include=None, discords=False, normalize=True):
     """
     Compute the k-dimensional matrix profile subspace for a given subsequence index and
     its nearest neighbor index
@@ -228,6 +283,11 @@ def subspace(T, m, subseq_idx, nn_idx, k, include=None, discords=False):
         When set to `True`, this reverses the distance profile to favor discords rather
         than motifs. Note that indices in `include` are still maintained and respected.
 
+    normalize : bool, default True
+        When set to `True`, this z-normalizes subsequences prior to computing distances.
+        Otherwise, this function gets re-routed to its complementary non-normalized
+        equivalent set in the `@core.non_normalized` function decorator.
+
     Returns
     -------
         S : ndarray
@@ -235,29 +295,14 @@ def subspace(T, m, subseq_idx, nn_idx, k, include=None, discords=False):
         with index equal to `motif_idx`
     """
     T, _, _ = core.preprocess(T, m)
-
     subseqs = core.z_norm(T[:, subseq_idx : subseq_idx + m], axis=1)
     neighbors = core.z_norm(T[:, nn_idx : nn_idx + m], axis=1)
+
     D = np.linalg.norm(subseqs - neighbors, axis=1)
 
-    if discords:
-        sorted_idx = D[::-1].argsort(axis=0, kind="mergesort")
-    else:
-        sorted_idx = D.argsort(axis=0, kind="mergesort")
-
-    # `include` processing occur here since we are dealing with indices, not distances
-    if include is not None:
-        include = _preprocess_include(include)
-        mask = np.in1d(sorted_idx, include)
-        include_idx = mask.nonzero()[0]
-        exclude_idx = (~mask).nonzero()[0]
-        sorted_idx[: include_idx.shape[0]], sorted_idx[include_idx.shape[0] :] = (
-            sorted_idx[include_idx],
-            sorted_idx[exclude_idx],
-        )
-
-    S = sorted_idx[: k + 1]
+    S = _subspace(D, k, include=include, discords=discords)
 
+    # MDL
     n_bit = 8
     bins = _inverse_norm()
     disc_subseqs = _discretize(subseqs[S], bins)
@@ -736,31 +781,13 @@ def _mstump(
 
     for idx in range(range_start, range_stop):
         _compute_multi_D(
-            d,
-            k,
-            idx,
-            D,
-            T,
-            m,
-            excl_zone,
-            M_T,
-            Σ_T,
-            QT_even,
-            QT_odd,
-            QT_first,
-            μ_Q,
-            σ_Q,
+            d, k, idx, D, T, m, excl_zone, M_T, Σ_T, QT_even, QT_odd, QT_first, μ_Q, σ_Q
         )
 
         # `include` processing must occur here since we are dealing with distances
         if include is not None:
             _apply_include(
-                D,
-                include,
-                restricted_indices,
-                unrestricted_indices,
-                mask,
-                tmp_swap,
+                D, include, restricted_indices, unrestricted_indices, mask, tmp_swap
             )
             start_row_idx = include.shape[0]
 
@@ -774,7 +801,8 @@ def _mstump(
     return P, I
 
 
-def mstump(T, m, include=None, discords=False):
+@core.non_normalized(maamp)
+def mstump(T, m, include=None, discords=False, normalize=True):
     """
     Compute the multi-dimensional z-normalized matrix profile
 
@@ -808,6 +836,11 @@ def mstump(T, m, include=None, discords=False):
         (i.e., discords) rather than smaller values (i.e., motifs). Note that indices
         in `include` are still maintained and respected.
 
+    normalize : bool, default True
+        When set to `True`, this z-normalizes subsequences prior to computing distances.
+        Otherwise, this function gets re-routed to its complementary non-normalized
+        equivalent set in the `@core.non_normalized` function decorator.
+
     Returns
     -------
     P : ndarray
@@ -843,7 +876,9 @@ def mstump(T, m, include=None, discords=False):
 
     d, n = T_B.shape
     k = n - m + 1
-    excl_zone = int(np.ceil(m / 4))  # See Definition 3 and Figure 3
+    excl_zone = int(
+        np.ceil(m / config.STUMPY_EXCL_ZONE_DENOM)
+    )  # See Definition 3 and Figure 3
 
     P = np.empty((d, k), dtype="float64")
     I = np.empty((d, k), dtype="int64")
