@@ -6,13 +6,20 @@ import logging
 
 import numpy as np
 
-from .mstump import _mstump, _get_first_mstump_profile, _get_multi_QT
-from . import core
+from .mstump import (
+    _mstump,
+    _get_first_mstump_profile,
+    _get_multi_QT,
+    _preprocess_include,
+)
+from . import core, config
+from .maamped import maamped
 
 logger = logging.getLogger(__name__)
 
 
-def mstumped(dask_client, T, m, include=None, discords=False):
+@core.non_normalized(maamped)
+def mstumped(dask_client, T, m, include=None, discords=False, normalize=True):
     """
     Compute the multi-dimensional z-normalized matrix profile with a distributed
     dask cluster
@@ -52,6 +59,11 @@ def mstumped(dask_client, T, m, include=None, discords=False):
         (i.e., discords) rather than smaller values (i.e., motifs). Note that indices
         in `include` are still maintained and respected.
 
+    normalize : bool, default True
+        When set to `True`, this z-normalizes subsequences prior to computing distances.
+        Otherwise, this function gets re-routed to its complementary non-normalized
+        equivalent set in the `@core.non_normalized` function decorator.
+
     Returns
     -------
     P : ndarray
@@ -83,15 +95,13 @@ def mstumped(dask_client, T, m, include=None, discords=False):
     core.check_window_size(m, max_size=min(T_A.shape[1], T_B.shape[1]))
 
     if include is not None:
-        include = np.asarray(include)
-        _, idx = np.unique(include, return_index=True)
-        if include.shape[0] != idx.shape[0]:  # pragma: no cover
-            logger.warning("Removed repeating indices in `include`")
-            include = include[np.sort(idx)]
+        include = _preprocess_include(include)
 
     d, n = T_B.shape
     k = n - m + 1
-    excl_zone = int(np.ceil(m / 4))  # See Definition 3 and Figure 3
+    excl_zone = int(
+        np.ceil(m / config.STUMPY_EXCL_ZONE_DENOM)
+    )  # See Definition 3 and Figure 3
 
     P = np.empty((d, k), dtype="float64")
     I = np.empty((d, k), dtype="int64")
