@@ -5,10 +5,11 @@
 import logging
 
 import numpy as np
-from numba import njit, prange, config
+from numba import njit, prange
+import numba
 
-from . import core
-from stumpy.config import STUMPY_D_SQUARED_THRESHOLD
+from . import core, config
+from .aamp import aamp
 
 logger = logging.getLogger(__name__)
 
@@ -342,7 +343,7 @@ def _stump(
     n_A = T_A.shape[0]
     n_B = T_B.shape[0]
     l = n_A - m + 1
-    n_threads = config.NUMBA_NUM_THREADS
+    n_threads = numba.config.NUMBA_NUM_THREADS
     ρ = np.full((n_threads, l, 3), -np.inf)
     I = np.full((n_threads, l, 3), -1, np.int64)
 
@@ -411,18 +412,19 @@ def _stump(
     # Convert pearson correlations to distances
     D = np.abs(2 * m * (1 - ρ[0, :, :]))
     for i in prange(D.shape[0]):
-        if D[i, 0] < STUMPY_D_SQUARED_THRESHOLD:
+        if D[i, 0] < config.STUMPY_D_SQUARED_THRESHOLD:
             D[i, 0] = 0.0
-        if D[i, 1] < STUMPY_D_SQUARED_THRESHOLD:
+        if D[i, 1] < config.STUMPY_D_SQUARED_THRESHOLD:
             D[i, 1] = 0.0
-        if D[i, 2] < STUMPY_D_SQUARED_THRESHOLD:
+        if D[i, 2] < config.STUMPY_D_SQUARED_THRESHOLD:
             D[i, 2] = 0.0
     P = np.sqrt(D)
 
     return P[:, :], I[0, :, :]
 
 
-def stump(T_A, m, T_B=None, ignore_trivial=True):
+@core.non_normalized(aamp)
+def stump(T_A, m, T_B=None, ignore_trivial=True, normalize=True):
     """
     Compute the z-normalized matrix profile
 
@@ -447,6 +449,11 @@ def stump(T_A, m, T_B=None, ignore_trivial=True):
         Set to `True` if this is a self-join. Otherwise, for AB-join, set this
         to `False`. Default is `True`.
 
+    normalize : bool, default True
+        When set to `True`, this z-normalizes subsequences prior to computing distances.
+        Otherwise, this function gets re-routed to its complementary non-normalized
+        equivalent set in the `@core.non_normalized` function decorator.
+
     Returns
     -------
     out : ndarray
@@ -548,7 +555,7 @@ def stump(T_A, m, T_B=None, ignore_trivial=True):
     n_B = T_B.shape[0]
     l = n_A - m + 1
 
-    excl_zone = int(np.ceil(m / 4))
+    excl_zone = int(np.ceil(m / config.STUMPY_EXCL_ZONE_DENOM))
     out = np.empty((l, 4), dtype=object)
 
     if ignore_trivial:
