@@ -6,13 +6,15 @@ import logging
 
 import numpy as np
 
-from . import core
+from . import core, config
 from .stump import _stump
+from .aamped import aamped
 
 logger = logging.getLogger(__name__)
 
 
-def stumped(dask_client, T_A, m, T_B=None, ignore_trivial=True):
+@core.non_normalized(aamped)
+def stumped(dask_client, T_A, m, T_B=None, ignore_trivial=True, normalize=True):
     """
     Compute the z-normalized matrix profile with a distributed dask cluster
 
@@ -43,6 +45,11 @@ def stumped(dask_client, T_A, m, T_B=None, ignore_trivial=True):
         Set to `True` if this is a self-join. Otherwise, for AB-join, set this
         to `False`. Default is `True`.
 
+    normalize : bool, default True
+        When set to `True`, this z-normalizes subsequences prior to computing distances.
+        Otherwise, this function gets re-routed to its complementary non-normalized
+        equivalent set in the `@core.non_normalized` function decorator.
+
     Returns
     -------
     out : ndarray
@@ -148,7 +155,7 @@ def stumped(dask_client, T_A, m, T_B=None, ignore_trivial=True):
     n_B = T_B.shape[0]
     l = n_A - m + 1
 
-    excl_zone = int(np.ceil(m / 4))
+    excl_zone = int(np.ceil(m / config.STUMPY_EXCL_ZONE_DENOM))
     out = np.empty((l, 4), dtype=object)
 
     hosts = list(dask_client.ncores().keys())
